@@ -1458,7 +1458,7 @@ def return_global_counterfactuals(ac_explainer, data, y_data, s_star, n_star, x_
     counterfactuals_samples_sdp = []
     counterfactuals_samples_w = []
 
-    for i in range(N):
+    for i in tqdm(range(N)):
         suf_leaf, _, _, _, _ = get_compatible_leaf(ac_explainer, data[i], 1 - y_data[i], s_star[i], n_star[i], w[i],
                                                    x_train, y_train, pi=pi_level, acc_level=acc_level)
         suf_leaves.append(suf_leaf)
@@ -1504,7 +1504,7 @@ def return_ge_global_counterfactuals(ac_explainer, data, y_data, s_star, n_star,
     counterfactuals_samples_sdp = []
     counterfactuals_samples_w = []
 
-    for i in range(N):
+    for i in tqdm(range(N)):
         suf_leaf, _, _, _, _ = get_compatible_leaf(ac_explainer, data[i], 1 - y_data[i], s_star[i], n_star[i], w[i],
                                                    x_train, y_train, pi=pi_level, acc_level=acc_level)
         suf_leaves.append(suf_leaf)
@@ -1595,7 +1595,8 @@ def load_model(name='{}'.format(dataset)):
 
 class RunExperiments:
 
-    def __init__(self, acv_explainer, x_train, x_test, y_train, y_test, columns_name):
+    def __init__(self, acv_explainer, x_train, x_test, y_train, y_test, columns_name, model=None):
+        self.model = None
         self.columns_name = columns_name
         self.acv_explainer = acv_explainer
         self.x_train = x_train
@@ -1611,7 +1612,9 @@ class RunExperiments:
         self.dist_local = None
         self.score_local = None
         self.errs_local = None
+        self.errs_local_original = None
         self.accuracy_local = None
+        self.accuracy_local_original = None
         self.coverage_local = None
         self.sdp_importance_se, self.sdp_index_se, self.size_se, self.sdp_se = None, None, None, None
         self.S_star_se, self.N_star_se = None, None
@@ -1624,7 +1627,9 @@ class RunExperiments:
         self.dist_regional = None
         self.score_regional = None
         self.errs_regional = None
+        self.errs_regional_original = None
         self.accuracy_regional = None
+        self.accuracy_regional_original = None
         self.coverage_regional = None
 
     def run_local_divergent_set(self, x, y, stop=True, pi_level=0.8):
@@ -1659,7 +1664,7 @@ class RunExperiments:
         self.dist_local = []
         self.score_local = []
         self.errs_local = []
-
+        self.errs_local_original = []
         for i in tqdm(range(x.shape[0])):
             if len(self.counterfactuals_samples_local[i]) != 0:
                 a, sco = simulated_annealing(outlier_score, x[i], self.S_star_local[i], self.x_train,
@@ -1671,7 +1676,13 @@ class RunExperiments:
                 self.errs_local.append(
                     self.acv_explainer.predict(self.dist_local[-1].reshape(1, -1)) != self.acv_explainer.predict(
                         x[i].reshape(1, -1)))
+                if self.model != None:
+                    self.errs_local_original.append(
+                        self.model.predict(self.dist_local[-1].reshape(1, -1)) != self.model.predict(
+                            x[i].reshape(1, -1)))
+
         self.accuracy_local = np.mean(self.errs_local)
+        self.accuracy_local_original = np.mean(self.errs_local_original)
         self.coverage_local = len(self.errs_local) / x.shape[0]
 
     def run_sufficient_rules(self, x_rule, y_rule, pi_level=0.9):
@@ -1722,11 +1733,13 @@ class RunExperiments:
         self.dist_regional = []
         self.score_regional = []
         self.errs_regional = []
-
+        self.errs_regional_original = []
+        nb = 0
         for i in range(self.x_rules.shape[0]):
             if len(self.counterfactuals_samples_regional[i]) != 0:
                 x_in = np.prod([(self.x_test[:, s] <= self.rules[i, s, 1]) * (self.x_test[:, s] > self.rules[i, s, 0])
                                 for s in range(self.x_train.shape[1])], axis=0).astype(bool)
+                nb += np.sum(x_in) if np.sum(x_in) <= max_obs else max_obs
                 print('observations in rule = {}'.format(np.sum(x_in)))
                 if np.sum(x_in) > 0:
                     for xi in tqdm(self.x_test[x_in][:max_obs]):
@@ -1738,9 +1751,57 @@ class RunExperiments:
                         self.score_regional.append(sco)
                         self.errs_regional.append(self.acv_explainer.predict(
                             self.dist_regional[-1].reshape(1, -1)) != self.acv_explainer.predict(xi.reshape(1, -1)))
+                        if self.model != None:
+                            self.errs_regional_original.append(self.model.predict(
+                                self.dist_regional[-1].reshape(1, -1)) != self.model.predict(xi.reshape(1, -1)))
 
         self.accuracy_regional = np.mean(self.errs_regional)
-        self.coverage_regional = len(self.errs_regional) / self.x_rules.shape[0]
+        self.accuracy_regional_original = np.mean(self.errs_regional_original)
+        self.coverage_regional = len(self.errs_regional) / nb
+
+    def run_sampling_regional_counterfactuals_alltests(self, max_obs=2, batch=1000, max_iter=1000, temp=0.5):
+        print('### Sampling using the regional counterfactual rules')
+
+        outlier_score = lambda x: self.isolation.decision_function(x)
+
+        self.dist_regional = []
+        self.score_regional = []
+        self.errs_regional = []
+        self.errs_regional_original = []
+        x_test_pb = []
+
+        for i in range(self.x_rules.shape[0]):
+            x_in = np.prod([(self.x_test[:, s] <= self.rules[i, s, 1]) * (self.x_test[:, s] > self.rules[i, s, 0])
+                            for s in range(self.x_train.shape[1])], axis=0)
+            if len(self.counterfactuals_samples_regional[i]) != 0:
+                x_in = np.max(self.counterfactuals_samples_sdp_regional[i]) * x_in
+            else:
+                x_in = 0 * x_in
+            x_test_pb.append(x_in)
+
+        x_test_pb = np.array(x_test_pb)
+        best_counterfactuals = np.argmax(x_test_pb, axis=0)
+
+        for i in tqdm(range(self.x_test.shape[0])):
+            xi = self.x_test[i]
+            best_id = best_counterfactuals[i]
+            if len(self.counterfactuals_samples_regional[best_id]) != 0:
+                a, sco = simulated_annealing(outlier_score, xi, self.S_star_regional[best_id], self.x_train,
+                                             self.counterfactuals_samples_regional[best_id][
+                                                 np.argmax(self.counterfactuals_samples_sdp_regional[best_id])][0],
+                                             batch, max_iter, temp)
+                self.dist_regional.append(np.squeeze(a))
+                self.score_regional.append(sco)
+                self.errs_regional.append(self.acv_explainer.predict(
+                    self.dist_regional[-1].reshape(1, -1)) != self.acv_explainer.predict(xi.reshape(1, -1)))
+
+                if self.model != None:
+                    self.errs_regional_original.append(self.model.predict(
+                        self.dist_regional[-1].reshape(1, -1)) != self.model.predict(xi.reshape(1, -1)))
+
+        self.accuracy_regional = np.mean(self.errs_regional)
+        self.accuracy_regional_original = np.mean(self.errs_regional_original)
+        self.coverage_regional = len(self.errs_regional) / self.x_test.shape[0]
 
     def show_global_counterfactuals(self):
 
